@@ -283,7 +283,7 @@ class Agent:
         stream: bool = False,
         allow_tool: bool = True,
         force_tool: bool = True,
-        disable_thinking: bool = False,
+        thinking_budget: int | None = None,
         **kwargs,
     ) -> AsyncGenerator[MessageContent | Message | list[Message], None]:
         """Process a single turn in the conversation.
@@ -296,7 +296,7 @@ class Agent:
             stream: Boolean flag for whether to yield partial message chunks from the API or not.
             allow_tool: Boolean flag for whether to fully interpret tool calls/results in the history.
             force_tool: Boolean flag for whether to force the use of tools.
-            disable_thinking: Boolean flag for whether to disable thinking.
+            thinking_budget: Budget for thinking tokens.
             **kwargs: Additional keyword arguments to pass to the client.
 
         Yields:
@@ -337,7 +337,7 @@ class Agent:
                 system=system,
                 tools=list(self.toolbox.values()),
                 allow_tool=allow_tool,
-                disable_thinking=disable_thinking,
+                thinking_budget=thinking_budget,
                 force_tool=force_tool,
                 **kwargs,
             ):
@@ -355,7 +355,7 @@ class Agent:
                 system=system,
                 tools=list(self.toolbox.values()),
                 force_tool=force_tool,
-                disable_thinking=disable_thinking,
+                thinking_budget=thinking_budget,
                 **kwargs,
             )
 
@@ -409,7 +409,7 @@ class Agent:
         stream: bool = False,
         allow_tool: bool = True,
         force_tool: bool = True,
-        disable_thinking: bool = False,
+        thinking_budget: int | None = None,
         **kwargs,
     ) -> AsyncGenerator[MessageContent | Message, None]:
         """Generate an asynchronous stream of agent responses and tool invocations.
@@ -422,7 +422,7 @@ class Agent:
             stream: Boolean flag for whether to yield partial message chunks from the inner API or not.
             allow_tool: Boolean flag for whether to fully interpret tool calls/results in the history.
             force_tool: Boolean flag for whether to force the use of tools.
-            disable_thinking: Boolean flag for whether to disable thinking.
+            thinking_budget: Budget for thinking tokens.
             **kwargs: Additional keyword arguments to pass to the client.
 
         Yields:
@@ -450,7 +450,7 @@ class Agent:
                 stream=stream,
                 allow_tool=allow_tool,
                 force_tool=force_tool,
-                disable_thinking=disable_thinking,
+                thinking_budget=thinking_budget,
                 **kwargs,
             ):
                 # This is a partial message chunk.
