@@ -93,7 +93,7 @@ class AnthropicClient(Client):
         *,
         system: str | SystemMessage = "",
         tools: list[Tool] | None = None,
-        disable_thinking: bool = False,
+        thinking_budget: int | None = None,
         temperature: float = 1.0,
         force_tool: bool = True,
         allow_tool: bool = True,
@@ -133,7 +133,7 @@ class AnthropicClient(Client):
             temperature=temperature,
             force_tool=force_tool,
             allow_tool=allow_tool,
-            disable_thinking=disable_thinking,
+            thinking_budget=thinking_budget,
             stream=False,
         )
         agent = kwargs.get("agent")
@@ -167,7 +167,7 @@ class AnthropicClient(Client):
         system: str | SystemMessage = "",
         tools: list[Tool] | None = None,
         allow_tool: bool = True,
-        disable_thinking: bool = False,
+        thinking_budget: int | None = None,
         temperature: float = 1.0,
         force_tool: bool = True,
         **kwargs,
@@ -193,7 +193,7 @@ class AnthropicClient(Client):
             temperature=temperature,
             force_tool=force_tool,
             allow_tool=allow_tool,
-            disable_thinking=disable_thinking,
+            thinking_budget=thinking_budget,
             stream=True,
         )
         agent = kwargs.get("agent")
@@ -312,7 +312,7 @@ class AnthropicClient(Client):
         force_tool: bool,
         allow_tool: bool,
         stream: bool,
-        disable_thinking: bool,
+        thinking_budget: int | None = None,
     ) -> dict:
         """Get the common params to send to the model.
 
@@ -324,7 +324,7 @@ class AnthropicClient(Client):
             force_tool: Whether to force the model to use the tools.
             allow_tool: Whether to allow tool calls in the content.
             stream: Whether to stream the response.
-            disable_thinking: Whether to disable thinking.
+            thinking_budget: How many tokens are in the thinking budget (0 if it should be disabled).
 
         Returns:
             The common params to send to the model.
@@ -339,7 +339,7 @@ class AnthropicClient(Client):
             "model": self.model,
             "temperature": temperature,
             "extra_headers": self.extra_headers,
-            **({"thinking": {"type": "disabled"}} if disable_thinking else {}),
+            **({"thinking": {"type": "disabled"}} if (thinking_budget == 0) else {}),
         }
 
         # Only add system message if it's not empty
