@@ -9,9 +9,7 @@ from dataclasses import InitVar, dataclass, field
 from pathlib import Path
 from typing import Any
 
-import google.auth.transport.requests  # noqa: F401
 from google import genai
-from google.auth import default  # noqa: F401
 from google.genai import types
 from google.genai._api_client import BaseApiClient
 from google.genai.types import (
@@ -24,7 +22,7 @@ from google.oauth2 import service_account
 from pydantic import BaseModel
 
 from flexai.llm.client import Client
-from flexai.llm.exceptions import StructuredResponseException
+from flexai.llm.exceptions import NoContentException, StructuredResponseException
 from flexai.message import (
     AIMessage,
     DataBlock,
@@ -44,6 +42,14 @@ from flexai.tool import Tool
 
 
 def get_tool_call(function_call) -> ToolCall:
+    """Convert a Gemini function call to a ToolCall object.
+
+    Args:
+        function_call: The Gemini function call object containing id, name, and args.
+
+    Returns:
+        A ToolCall object with the function call information.
+    """
     return ToolCall(
         id=function_call.id or str(uuid.uuid4()),
         name=function_call.name,
@@ -126,7 +132,7 @@ class GeminiClient(Client):
     api_key: InitVar[str] = field(default=os.environ.get("GEMINI_API_KEY", ""))
 
     # The client to use for interacting with the model.
-    client: genai.client.AsyncClient | None = None
+    _client: genai.client.AsyncClient | None = None
 
     # The base URL for the Gemini API.
     base_url: InitVar[str] = field(
@@ -139,7 +145,7 @@ class GeminiClient(Client):
     # The model to use for the client.
     model: str = os.getenv("GEMINI_MODEL") or "gemini-2.5-pro-preview-06-05"
 
-    # Default thinking budget for LLM calls
+    # Default thinking budget for LLM calls (0 if disabled, None if we don't care)
     default_thinking_budget: int | None = None
 
     # Project ID for Vertex AI (required when using Vertex AI or global endpoint)
@@ -151,13 +157,29 @@ class GeminiClient(Client):
     # Whether to use Vertex AI instead of direct API
     use_vertex: bool = False
 
-    def __post_init__(self, api_key, base_url, **kwargs):
-        use_vertex = kwargs.get("use_vertex", self.use_vertex)
-        credential_file_path = kwargs.get("credential_file_path", "")
+    @staticmethod
+    def _get_credentials(
+        use_vertex: bool, location: str, project_id: str, credential_file_path: str
+    ) -> service_account.Credentials:
+        """Get Google Cloud credentials for Vertex AI or global endpoint access.
 
-        if use_vertex or self.location == "global":
+        Args:
+            use_vertex: Whether using Vertex AI endpoint.
+            location: The location/region for the endpoint.
+            project_id: Google Cloud project ID.
+            credential_file_path: Path to service account credentials file.
+
+        Returns:
+            Google Cloud service account credentials.
+
+        Raises:
+            ValueError: If project_id is missing when required or if authentication fails.
+        """
+        credentials = None
+
+        if use_vertex or location == "global":
             # Using Vertex AI or global endpoint
-            if not self.project_id:
+            if not project_id:
                 raise ValueError(
                     "project_id is required when using Vertex AI or global endpoint. "
                     "Set GOOGLE_PROJECT_ID environment variable or pass project_id parameter."
@@ -171,7 +193,7 @@ class GeminiClient(Client):
             # Handle authentication
             if credential_file_path and Path(credential_file_path).exists():
                 # Use service account file
-                creds = service_account.Credentials.from_service_account_file(
+                credentials = service_account.Credentials.from_service_account_file(
                     credential_file_path, scopes=scopes
                 )
             else:
@@ -180,34 +202,101 @@ class GeminiClient(Client):
                     from google.auth import default
 
                     creds, _ = default(scopes=scopes)
+                    credentials = creds
+
                 except Exception as e:
                     raise ValueError(
                         "Failed to load default credentials. Either set up Application Default Credentials "
                         "or provide credential_file_path."
                     ) from e
 
+        if credentials is None:
+            raise ValueError(
+                "No credentials were fetched for an unknown reason.",
+            )
+
+        return credentials  # pyright: ignore[reportReturnType]
+
+    @staticmethod
+    @functools.lru_cache
+    def _get_vertex_client(location: str, project_id: str, credential_file_path: str):
+        """Get a cached Vertex AI client instance.
+
+        Args:
+            location: The location/region for Vertex AI.
+            project_id: Google Cloud project ID.
+            credential_file_path: Path to service account credentials file.
+
+        Returns:
+            A cached AsyncClient configured for Vertex AI.
+        """
+        return genai.client.AsyncClient(
+            api_client=BaseApiClient(
+                vertexai=True,
+                credentials=GeminiClient._get_credentials(
+                    use_vertex=True,
+                    location=location,
+                    project_id=project_id,
+                    credential_file_path=credential_file_path,
+                ),
+                location=location,
+                project=project_id,
+            )
+        )
+
+    @staticmethod
+    @functools.lru_cache
+    def _get_client(api_key: str):
+        """Get a cached direct API client instance.
+
+        Args:
+            api_key: The Gemini API key for direct access.
+
+        Returns:
+            A cached AsyncClient configured for direct API access.
+        """
+        return genai.client.AsyncClient(api_client=BaseApiClient(api_key=api_key))
+
+    def __post_init__(self, api_key, base_url, **kwargs):
+        use_vertex = kwargs.get("use_vertex", self.use_vertex)
+        credential_file_path = kwargs.get("credential_file_path", "")
+
+        if use_vertex:
             object.__setattr__(
                 self,
-                "client",
-                genai.client.AsyncClient(
-                    api_client=BaseApiClient(
-                        vertexai=True,
-                        credentials=creds,
-                        location=self.location,
-                        project=self.project_id,
-                    )
+                "_client",
+                GeminiClient._get_vertex_client(
+                    location=self.location,
+                    project_id=self.project_id,
+                    credential_file_path=credential_file_path,
                 ),
             )
         else:
             # Using direct API
             object.__setattr__(
                 self,
-                "client",
-                genai.client.AsyncClient(api_client=BaseApiClient(api_key=api_key)),
+                "_client",
+                GeminiClient._get_client(
+                    api_key=api_key,
+                ),
             )
 
+    @property
+    def client(self) -> genai.client.AsyncClient:
+        if self._client is None:
+            raise ValueError("No Gemini Client Configured.")
+        return self._client
+
     @staticmethod
     def format_tool(tool: Tool) -> FunctionDeclaration:
+        """Convert a FlexAI Tool to a Gemini FunctionDeclaration.
+
+        Args:
+            tool: The FlexAI Tool object to convert.
+
+        Returns:
+            A Gemini FunctionDeclaration with the tool's name, description, and parameters.
+        """
         return FunctionDeclaration(
             name=tool.name,
             description=tool.description,
@@ -222,6 +311,14 @@ class GeminiClient(Client):
 
     @staticmethod
     def _extract_content_from_part_object(part_object: types.Part):
+        """Extract content from a Gemini Part object.
+
+        Args:
+            part_object: The Gemini Part object containing text or function call data.
+
+        Yields:
+            TextBlock, ThoughtBlock, or ToolCall objects extracted from the part.
+        """
         if part_object.text is not None:
             if part_object.thought is not None:
                 yield ThoughtBlock(
@@ -241,6 +338,19 @@ class GeminiClient(Client):
         content: str | MessageContent | Sequence[MessageContent],
         name_context: dict,
     ):
+        """Format message content for Gemini API.
+
+        Args:
+            content: The message content to format (string, MessageContent, or sequence).
+            name_context: Dictionary to track tool call names by ID.
+
+        Returns:
+            Formatted content structure for Gemini API.
+
+        Raises:
+            ValueError: If unsupported content type or tool call context issues.
+            TypeError: If tool result format is invalid.
+        """
         if isinstance(content, str):
             return [{"text": content}]
 
@@ -311,13 +421,29 @@ class GeminiClient(Client):
         tools: list[Tool] | None,
         force_tool: bool,
         include_thoughts: bool,
-        disable_thinking: bool,
         thinking_budget: int | None,
         use_url_context: bool = False,
         use_google_search: bool = False,
         google_search_dynamic_threshold: float | None = None,
         **kwargs,
     ):
+        """Build parameters for Gemini API request.
+
+        Args:
+            messages: List of conversation messages.
+            system: System message or string.
+            tools: Available tools for the model to use.
+            force_tool: Whether to force tool usage.
+            include_thoughts: Whether to include thinking in response.
+            thinking_budget: Token budget for thinking (None for unlimited).
+            use_url_context: Whether to enable URL context retrieval.
+            use_google_search: Whether to enable Google Search integration.
+            google_search_dynamic_threshold: Threshold for dynamic search retrieval.
+            kwargs: Additional parameters including structured output models.
+
+        Returns:
+            Dictionary of formatted parameters for Gemini API call.
+        """
         name_context = {}
 
         formatted_messages = [
@@ -345,9 +471,6 @@ class GeminiClient(Client):
 
         thinking_args = {}
 
-        if disable_thinking:
-            thinking_budget = 0
-
         if thinking_budget is not None:
             thinking_args["thinking_budget"] = thinking_budget
 
@@ -435,7 +558,6 @@ class GeminiClient(Client):
         system: str | SystemMessage = "",
         tools: list[Tool] | None = None,
         force_tool: bool = True,
-        disable_thinking: bool = False,
         thinking_budget: int | None = None,
         include_thoughts: bool = False,
         use_url_context: bool = False,
@@ -443,20 +565,42 @@ class GeminiClient(Client):
         google_search_dynamic_threshold: float | None = None,
         **kwargs,
     ) -> AIMessage:
-        if not self.client:
-            raise ValueError("GeminiClient is not initialized.")
+        """Get a chat response from the Gemini model.
+
+        Args:
+            messages: List of conversation messages.
+            system: System message to set AI behavior.
+            tools: Available tools for the model to use.
+            force_tool: Whether to force tool usage when tools are available.
+            thinking_budget: Token budget for thinking (None uses default).
+            include_thoughts: Whether to include thinking in the response.
+            use_url_context: Whether to enable URL context retrieval.
+            use_google_search: Whether to enable Google Search integration.
+            google_search_dynamic_threshold: Threshold for dynamic search retrieval.
+            kwargs: Additional parameters including structured output models.
+
+        Returns:
+            An AIMessage containing the model's response and usage information.
+
+        Raises:
+            ValueError: If the model doesn't respond with usage metadata.
+            NoContentException: If the model doesn't return any content.
+        """
         extra_param_args = {}
         if "model" in kwargs:
             extra_param_args["model"] = kwargs["model"]
 
         # If this client has a default thinking budget set, use that if one wasn't specified here
-        thinking_budget = thinking_budget or self.default_thinking_budget
+        thinking_budget = (
+            thinking_budget
+            if thinking_budget is not None
+            else self.default_thinking_budget
+        )
         params = self._get_params(
             messages=messages,
             system=system,
             tools=tools,
             force_tool=force_tool,
-            disable_thinking=disable_thinking,
             thinking_budget=thinking_budget,
             include_thoughts=include_thoughts,
             use_url_context=use_url_context,
@@ -483,12 +627,15 @@ class GeminiClient(Client):
         )
         # Handle this case
         if not response_object.candidates or not response_object.candidates[0].content:
-            raise ValueError(
-                "Gemini did not respond with any content, and candidates is null."
+            raise NoContentException(
+                message="Gemini did not respond with any content, and the candidates are null.",
+                provider="gemini",
             )
         response_content_parts = response_object.candidates[0].content.parts
         if not response_content_parts:
-            raise ValueError("Gemini did not respond with any content.")
+            raise NoContentException(
+                message="Gemini did not respond with any content.", provider="gemini"
+            )
         formatted_content_parts: list[
             TextBlock | ThoughtBlock | ToolCall | URLContextBlock | GroundingBlock
         ] = []
@@ -528,7 +675,6 @@ class GeminiClient(Client):
         tools: list[Tool] | None = None,
         allow_tool: bool = True,
         force_tool: bool = True,
-        disable_thinking: bool = False,
         thinking_budget: int | None = None,
         include_thoughts: bool = False,
         use_url_context: bool = False,
@@ -536,22 +682,44 @@ class GeminiClient(Client):
         google_search_dynamic_threshold: float | None = None,
         **kwargs,
     ) -> AsyncGenerator[MessageContent | AIMessage, None]:
-        if not self.client:
-            raise ValueError("GeminiClient is not initialized.")
+        """Stream a chat response from the Gemini model.
 
+        Args:
+            messages: List of conversation messages.
+            system: System message to set AI behavior.
+            tools: Available tools for the model to use.
+            allow_tool: Whether to allow tool usage (kept for compatibility).
+            force_tool: Whether to force tool usage when tools are available.
+            thinking_budget: Token budget for thinking (None uses default).
+            include_thoughts: Whether to include thinking in the response.
+            use_url_context: Whether to enable URL context retrieval.
+            use_google_search: Whether to enable Google Search integration.
+            google_search_dynamic_threshold: Threshold for dynamic search retrieval.
+            kwargs: Additional parameters including structured output models.
+
+        Yields:
+            TextBlock, ThoughtBlock, ToolCall content as they're generated,
+            followed by a final AIMessage with complete response and usage data.
+
+        Raises:
+            ValueError: If the model doesn't respond with usage metadata.
+        """
         extra_param_args = {}
         if "model" in kwargs:
             extra_param_args["model"] = kwargs["model"]
 
         # If this client has a default thinking budget set, use that if one wasn't specified here
-        thinking_budget = thinking_budget or self.default_thinking_budget
+        thinking_budget = (
+            thinking_budget
+            if thinking_budget is not None
+            else self.default_thinking_budget
+        )
         usage = Usage()
         params = self._get_params(
             messages=messages,
             system=system,
             tools=tools,
             force_tool=force_tool,
-            disable_thinking=disable_thinking,
             thinking_budget=thinking_budget,
             include_thoughts=include_thoughts,
             use_url_context=use_url_context,
@@ -781,7 +949,6 @@ class GeminiClient(Client):
         model: type[BaseModel],
         system: str | SystemMessage = "",
         tools: list[Tool] | None = None,
-        disable_thinking: bool = False,
         thinking_budget: int | None = None,
         include_thoughts: bool = False,
         use_url_context: bool = False,
@@ -796,8 +963,7 @@ class GeminiClient(Client):
             model: The model to use for the response.
             system: Optional system message to set the behavior of the AI.
             tools: Tools to use in the response.
-            disable_thinking: Whether to disable thinking for this request.
-            thinking_budget: The thinking budget for this request.
+            thinking_budget: The thinking budget for this request (0 if it should be disabled).
             include_thoughts: Whether to include thoughts in the response.
             use_url_context: Whether to enable URL context for accessing web content.
             use_google_search: Whether to enable Google Search for grounding.
@@ -815,7 +981,6 @@ class GeminiClient(Client):
             system=system,
             tools=tools,
             force_tool=False,
-            disable_thinking=disable_thinking,
             thinking_budget=thinking_budget,
             include_thoughts=include_thoughts,
             use_url_context=use_url_context,
