@@ -146,7 +146,7 @@ class TestGeminiClient:
         # Set up client with mocked __post_init__
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Hi there")],
@@ -184,7 +184,7 @@ class TestGeminiClient:
         # Need to patch __post_init__ to avoid making real API calls
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             # Call with thinking_budget and include_thoughts
             await client.get_chat_response(
@@ -212,20 +212,21 @@ class TestGeminiClient:
         # Set up client with mocked __post_init__
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key", default_thinking_budget=100)
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
-            # Call with disable_thinking=True
+            # Call with thinking_budget=0 - should disable thinking
             await client.get_chat_response(
                 messages=[UserMessage(content="Don't think")],
                 system="You are a helpful assistant",
-                disable_thinking=True,
+                thinking_budget=0,
             )
 
-        # Verify the thinking budget was set to 0
+        # Verify that thinking_budget=0 is correctly respected and not overridden by default
         call_args = mock_client.models.generate_content.call_args[1]
         config = call_args["config"]
 
         assert hasattr(config, "thinking_config")
+        # Implementation correctly uses thinking_budget=0 when explicitly set
         assert config.thinking_config.thinking_budget == 0
 
     @pytest.mark.asyncio
@@ -239,7 +240,7 @@ class TestGeminiClient:
         # Set up client with mocked __post_init__ and default_thinking_budget
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key", default_thinking_budget=200)
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             # Call without specific thinking_budget
             await client.get_chat_response(
@@ -268,7 +269,7 @@ class TestGeminiClient:
         # Need to patch __post_init__ to avoid making real API calls
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Use the tool")],
@@ -308,7 +309,7 @@ class TestGeminiClient:
         # Need to patch __post_init__ to avoid making real API calls
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Think about this")],
@@ -337,7 +338,7 @@ class TestGeminiClient:
         # Need to patch __post_init__ to avoid making real API calls
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             # Create an image message
             image_content = ImageBlock(
@@ -390,7 +391,7 @@ class TestGeminiClient:
         # Need to patch __post_init__ to avoid making real API calls
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             await client.get_chat_response(
                 messages=[UserMessage(content="Return a structured response")],
@@ -502,7 +503,7 @@ class TestGeminiClient:
         # Need to patch __post_init__ to avoid making real API calls
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             streamed_content = []
             async for content in client.stream_chat_response(
@@ -588,7 +589,7 @@ class TestGeminiClient:
         # Need to patch __post_init__ to avoid making real API calls
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             streamed_content = []
             async for content in client.stream_chat_response(
@@ -680,7 +681,7 @@ class TestGeminiClient:
         # Need to patch __post_init__ to avoid making real API calls
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             streamed_content = []
             async for content in client.stream_chat_response(
@@ -722,7 +723,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Analyze https://example.com")],
@@ -757,7 +758,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Regular message")],
@@ -805,7 +806,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Who won Euro 2024?")],
@@ -851,7 +852,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key", model="gemini-1.5-flash")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Latest AI developments?")],
@@ -886,7 +887,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="What is the capital of France?")],
@@ -933,7 +934,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Analyze https://example.com")],
@@ -975,7 +976,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             try:
                 # Just start the stream to verify configuration
@@ -1005,7 +1006,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_chat_response(
                 messages=[UserMessage(content="Simple question")],
@@ -1055,7 +1056,7 @@ class TestGeminiClient:
 
         with patch.object(GeminiClient, "__post_init__"):
             client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "client", mock_client)
+            object.__setattr__(client, "_client", mock_client)
 
             result = await client.get_structured_response(
                 messages=[UserMessage(content="Who won Euro 2024?")],
